Public Class BriefCreator
    Private m_Pat As Patient
   
    Public Sub New(ByVal Pat As Patient, ByVal IstKurzBrief As Boolean)
        m_Pat = Pat



        Dim MinDatum As New Date(1860, 1, 1)
        Dim Fehlermeldungen As New List(Of String)
        Dim DiagnosenGueltig As Boolean = False
        For Each Diag As TextEintrag In Pat.Diagnosen
            If Not Diag.IsEmpty Then
                DiagnosenGueltig = True
                Exit For
            End If
        Next
        If Not DiagnosenGueltig Then
            Fehlermeldungen.Add("keine Diagnosen angegeben")
        End If

        If Fehlermeldungen.Count > 0 Then
            Dim Meldg As String = Join(Fehlermeldungen.ToArray, "," & vbCrLf)
            Meldg &= "."
            Meldg = Meldg.Substring(0, 1).ToUpper & Meldg.Substring(1)
            MessageBox.Show(Meldg, "Visite", MessageBoxButtons.OK, MessageBoxIcon.Information)
            Exit Sub
        End If

        Dim frm As New frm_EntlassungsMedikation(Pat)
        If frm.ShowDialog <> Windows.Forms.DialogResult.OK Then
            Exit Sub
        End If

        Dim Dat As New ParsedDate(frm.ValidatedDateTextBoxAufnahme.Text)
        Pat.Aufnahmedatum = Dat.Datum
        Dat = New ParsedDate(frm.ValidatedDateTextBoxEntlassung.Text)
        Pat.Entlassungsdatum = Dat.Datum
        Pat.Medikamente.Clear()
        For Each MV As MedikamentVerordnung In frm.Medi
            Dim Verord As New MedikamentVerordnung(Pat.Medikamente, MV)
            Pat.Medikamente.Add(Verord)
        Next


        Dim Wdapp As New Word.Application
        Dim Wddoc As Word.Document = Wdapp.Documents.Add(DataProvider.DataPath & "\Briefvorlage.dot")
        Wdapp.Visible = True



        Dim Aerzte As New List(Of Arzt)
        For Each Lbl As ArztLabel In Pat.Aerzte
            DataProvider.ArztProvider.SetActive(Lbl)
            Aerzte.Add(DataProvider.ArztProvider.ActiveEditArzt)
        Next


        Dim Rng As Word.Range
        For Each BM As Word.Bookmark In Wddoc.Bookmarks
            Rng = BM.Range
            If BM.Name = "Anschrift" Then
                If Aerzte.Count > 0 Then
                    Rng.Text = Aerzte(0).AdressAnsicht
                End If
            ElseIf BM.Name = "Datum" Then
                WriteText(Rng, False, Date.Today.ToShortDateString)
            ElseIf BM.Name = "Betreff" Then
                If IstKurzBrief Then WriteText(Rng, True, "Kurzbrief")
            ElseIf BM.Name = "Nachrichtlich" Then
                If Aerzte.Count > 1 Then
                    WriteText(Rng, False, "Nachrichtlich:" & vbCrLf)
                    For I As Integer = 1 To Aerzte.Count - 1
                        Dim Zus As String = Aerzte(I).AdressAnsicht.Replace(vbCrLf, ", ")
                        Zus = Zus.Replace(", , ", ", ")
                        WriteText(Rng, False, Zus & vbCrLf)
                    Next
                End If
            ElseIf BM.Name = "Brieftext" Then
                Dim Anrede As String
                If Aerzte.Count > 0 Then
                    Anrede = Aerzte(0).Briefanrede
                Else
                    Anrede = "Sehr geehrte Kollegen,"
                End If
                WriteText(Rng, False, Anrede & vbCrLf & vbCrLf)

                If Pat.Geschlecht = Geschlechter.maennlich Then
                    WriteText(Rng, False, "wir berichten ber den stationren Aufenthalt des Patienten ")
                Else
                    WriteText(Rng, False, "wir berichten ber den stationren Aufenthalt der Patientin ")
                End If
                WriteText(Rng, True, Pat.Nachname & ", " & Pat.Vorname & " ")
                WriteText(Rng, False, "geb. ")
                WriteText(Rng, True, Pat.Geburtsdatum.ToShortDateString & " ")
                WriteText(Rng, False, "im Zeitraum vom ")
                WriteText(Rng, True, Pat.Aufnahmedatum.ToShortDateString & " ")
                WriteText(Rng, False, "bis ")
                WriteText(Rng, True, Pat.Entlassungsdatum.ToShortDateString)
                WriteText(Rng, False, "." & vbCrLf & vbCrLf)
                WriteText(Rng, True, "Diagnosen" & vbNewLine)
                InsertBookmark(Rng, "DiagnosenStart")
                For Each DA As TextEintrag In Pat.Diagnosen
                    If Not DA.IsEmpty Then
                        WriteText(Rng, False, DA.Text & vbNewLine)
                    End If
                Next
                InsertBookmark(Rng, "DiagnosenEnde")

                WriteText(Rng, False, vbCrLf)

                WriteText(Rng, True, "Befunde" & vbNewLine)
                Dim ZText As String



                For Each DA As BefundTextEintrag In Pat.Befunde
                    If Not DA.IsEmpty Then
                        ZText = ""
                        Dim UEintrag As BefundStichwort = Nothing
                        For Each eintr As BefundStichwort In DataProvider.BefundStichworte
                            If eintr.KurzWort = DA.Kategorie Then
                                UEintrag = eintr
                                Exit For
                            End If
                        Next
                        If UEintrag Is Nothing Then
                            ZText &= " " & DA.Kategorie
                        Else
                            ZText &= " " & UEintrag.Wort
                        End If


                        If DA.Datum > MinDatum Then
                            ZText &= " am " & DA.Datum.ToShortDateString
                        End If
                        ZText = ZText.Trim
                        ZText &= ": "
                        WriteText(Rng, True, ZText)
                        WriteText(Rng, False, DA.Text & vbNewLine)
                    End If
                Next

                WriteText(Rng, False, vbCrLf)

                WriteText(Rng, True, "Verlauf" & vbNewLine)
                For Each Eintr As TextEintrag In Pat.FreiTexte
                    If Not Eintr.IsEmpty Then
                        WriteText(Rng, False, Eintr.Text & vbNewLine)
                    End If
                Next

                WriteText(Rng, False, vbCrLf & vbCrLf)
                Dim Medikamente As New List(Of MedikamentVerordnung)
                For Each Eintr As MedikamentVerordnung In frm.Medi
                    If Not Eintr.IsEmpty Then
                        Medikamente.Add(Eintr)
                    End If
                Next
                WriteText(Rng, True, "Entlassungsmedikation" & vbNewLine)
                If Medikamente.Count = 0 Then
                    WriteText(Rng, False, "keine nderung" & vbNewLine)
                Else

                    Dim Zeilen(Medikamente.Count - 1, 2) As String
                    Dim Zaehler As Integer = 0
                    For Each Med As MedikamentVerordnung In Medikamente
                        Zeilen(Zaehler, 0) = Med.Medikament
                        Zeilen(Zaehler, 1) = Med.MedGabeText
                        Zeilen(Zaehler, 2) = Med.Hinweise
                        Zaehler += 1
                    Next
                    InsertTable(Rng, Zeilen, True)


                End If
                WriteText(Rng, False, vbCrLf & vbCrLf)

                WriteText(Rng, False, "Mit freundlichen Grssen" & vbCrLf & vbCrLf & vbCrLf)

                Dim Unterschriften As New List(Of Unterschrift)
                For Each US As Unterschrift In DataProvider.Unterschriften
                    If Not US.IsEmpty Then
                        Unterschriften.Add(US)
                    End If
                Next
                If Unterschriften.Count > 0 Then
                    If IstKurzBrief Then
                        WriteText(Rng, False, Unterschriften(0).Name & vbCrLf)
                        WriteText(Rng, False, Unterschriften(0).Dienstgrad)
                    Else
                        Dim Zeilen(1, Unterschriften.Count - 1) As String
                        Dim Zaehler As Integer = 0
                        For Each US As Unterschrift In Unterschriften
                            Zeilen(0, Zaehler) = US.Name
                            Zeilen(1, Zaehler) = US.Dienstgrad
                            Zaehler += 1
                        Next
                        InsertTable(Rng, Zeilen, False)
                    End If
                End If

            End If
        Next

        Dim DiagStartIndex As Integer = 1
        For Each BM As Word.Bookmark In Wddoc.Bookmarks
            If BM.Name = "DiagnosenStart" Then
                Exit For
            End If
            DiagStartIndex += 1
        Next
        Dim DiagEndIndex As Integer = 1
        For Each BM As Word.Bookmark In Wddoc.Bookmarks
            If BM.Name = "DiagnosenEnde" Then
                Exit For
            End If
            DiagEndIndex += 1
        Next

        Rng = Wddoc.Range(Wddoc.Bookmarks.Item(DiagStartIndex).Range.Start, Wddoc.Bookmarks.Item(DiagEndIndex).Range.Start)
        Rng.ListFormat.ApplyBulletDefault()

        Dim BriefName As String = Pat.Nachname & " " & Pat.Vorname
        Dim Chars() As Char = IO.Path.GetInvalidFileNameChars
        For Each Bst As Char In Chars
            BriefName = BriefName.Replace(Bst, " ")
        Next

        Dim BasisPfad As String = DataProvider.BriefePath & "\"
        Dim ResultName As String = BriefName & ".doc"
        Dim Zahl As Integer = 0
        While IO.File.Exists(BasisPfad & ResultName)
            Zahl += 1
            ResultName = BriefName & Zahl.ToString & ".doc"
        End While

        Wddoc.SaveAs(BasisPfad & ResultName)
        Pat.Briefe.Add(ResultName)


        If Wddoc IsNot Nothing Then System.Runtime.InteropServices.Marshal.ReleaseComObject(Wddoc)
        If Wdapp IsNot Nothing Then System.Runtime.InteropServices.Marshal.ReleaseComObject(Wdapp)
        Wddoc = Nothing
        Wdapp = Nothing

        GC.Collect()
    End Sub
    Private Sub InsertTable(ByRef Rng As Word.Range, ByVal Contents As String(,), ByVal AutoFit As Boolean)
        Rng.Collapse(Word.WdCollapseDirection.wdCollapseEnd)

        Dim Tbl As Word.Table = Rng.Tables.Add(Rng, Contents.GetUpperBound(0) + 1, Contents.GetUpperBound(1) + 1)
        Tbl.Range.ParagraphFormat.SpaceBefore = CSng(0)
        Tbl.Borders.InsideLineStyle = Word.WdLineStyle.wdLineStyleNone
        Tbl.Borders.OutsideLineStyle = Word.WdLineStyle.wdLineStyleNone


        For K As Integer = 0 To Contents.GetUpperBound(0)
            For L As Integer = 0 To Contents.GetUpperBound(1)
                Tbl.Cell(K + 1, L + 1).Range.Text = Contents(K, L)
            Next
        Next
        If AutoFit Then Tbl.AutoFitBehavior(Word.WdAutoFitBehavior.wdAutoFitContent)
        Rng = Tbl.Range


    End Sub

    Private Sub WriteText(ByVal Rng As Word.Range, ByVal Bold As Boolean, ByVal Text As String)
        Rng.Collapse(Word.WdCollapseDirection.wdCollapseEnd)
        Rng.Text = Text
        Rng.Font.Bold = Bold
    End Sub
    Private Sub InsertBookmark(ByVal Rng As Word.Range, ByVal Name As String)
        Rng.Collapse(Word.WdCollapseDirection.wdCollapseEnd)
        With Rng.Bookmarks
            .Add(Range:=Rng, Name:=Name)
            .ShowHidden = False
        End With
    End Sub

End Class
