Public Class ColorStringPainter
    Inherits System.Windows.Forms.Control
    Private WithEvents m_source As ITempItemList
    Private m_BMP As Bitmap
    Private WithEvents m_VBar As VScrollBar
    Private m_Group As ColorStringGroup
    Private m_GroupHeight As Single = 0.0F
    Private m_ScrollAllowed As Boolean = True

    WriteOnly Property Source() As ITempItemList
        Set(ByVal value As ITempItemList)
            m_source = value
            ReCreateGroup()
            GetGroupHeight()

            m_BMP = Nothing
            Invalidate()
        End Set
    End Property

    Private Sub ReCreateGroup()
        If m_source IsNot Nothing Then
            m_Group = New ColorStringGroup(Me.Font)
            Dim CStrings As List(Of ColorString) = m_source.GetColorStrings
            For Each CString As ColorString In CStrings
                m_Group.Add(CString)
            Next
        Else
            m_Group = Nothing
        End If
    End Sub
    Private Sub GetGroupHeight()
        If m_Group Is Nothing Then
            m_GroupHeight = 0.0F
        Else
            Dim g As Graphics = Me.CreateGraphics
            m_GroupHeight = m_Group.GetHeight(g, CSng(Me.Width - m_VBar.Width - 2))
        End If
        ReCalculateScrollBar()
    End Sub
    Private Sub ReCalculateScrollBar()
        m_ScrollAllowed = False
        m_VBar.Value = 0
        If m_GroupHeight > Me.Height - 2 Then
            m_VBar.Enabled = True
            m_VBar.LargeChange = Math.Max(Me.Height - 2, 0)

            m_VBar.Maximum = m_GroupHeight
            'Der Wert einer Bildlaufleiste kann durch einen Benutzervorgang whrend der Laufzeit 
            'nicht seinen grtmglichen Wert erreichen. Der grtmgliche Wert ist gleich des 
            'Maximum-Eigenschaftenwerts minus des LargeChange-Eigenschaftenwerts plus 1. 
            'Der Hchstwert kann nur programmatisch erreicht werden.

        Else
            m_VBar.Enabled = False
        End If

        m_ScrollAllowed = True
    End Sub



    Public Sub New()

        ' Fgen Sie Initialisierungen nach dem InitializeComponent()-Aufruf hinzu.
        Me.SetStyle(ControlStyles.DoubleBuffer _
                   Or ControlStyles.UserPaint _
                   Or ControlStyles.AllPaintingInWmPaint, _
                   True)
        m_VBar = New VScrollBar
        m_VBar.Enabled = False
        m_VBar.Height = Me.Height - 2
        m_VBar.Location = New Point(Me.Width - m_VBar.Width - 1, 1)
        m_VBar.Anchor = AnchorStyles.Top Or AnchorStyles.Right Or AnchorStyles.Bottom
        Me.Controls.Add(m_VBar)
    End Sub

    Protected Overrides Sub OnPaint(ByVal pe As System.Windows.Forms.PaintEventArgs)
        MyBase.OnPaint(pe)

        'Benutzerdefinierten Zeichnungscode hier einfgen
        If m_BMP Is Nothing Then
            m_BMP = New Bitmap(Me.Width, Me.Height)
            Dim g As Graphics = Graphics.FromImage(m_BMP)
            If Me.Enabled Then
                g.Clear(Color.White)
            End If

            If m_Group IsNot Nothing Then
                m_Group.Print(g, 1.0F, 1.0F - CSng(Me.m_VBar.Value), CSng(Me.Width - 2 - Me.m_VBar.Width))
            End If

            ControlPaint.DrawVisualStyleBorder(g, New Rectangle(0, 0, Me.Width - 1, Me.Height - 1))
        End If
        pe.Graphics.DrawImage(m_BMP, New Rectangle(0, 0, Me.Width, Me.Height))

    End Sub


    Private Sub FreiTextPainter_EnabledChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.EnabledChanged
        m_BMP = Nothing
        Invalidate()
    End Sub

    Private Sub FreiTextPainter_Resize(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Resize
        GetGroupHeight()

        m_BMP = Nothing
        Invalidate()
    End Sub

    Private Sub m_source_Changed(ByVal PropertyName As String) Handles m_source.Changed
        ReCreateGroup()
        GetGroupHeight()

        m_BMP = Nothing
        Invalidate()
    End Sub

    Private Sub m_VBar_Scroll(ByVal sender As Object, ByVal e As System.Windows.Forms.ScrollEventArgs) Handles m_VBar.Scroll
        If Not m_ScrollAllowed Then Exit Sub

        m_BMP = Nothing
        Invalidate()
    End Sub
End Class
