Public Class EintragsEditor
    Private WithEvents m_Source As ITempItemList
    Private m_EventsEnabled As Boolean = True
 
    Property Source() As ITempItemList
        Get
            Return m_Source
        End Get
        Set(ByVal value As ITempItemList)
            If m_Source IsNot value Then
                Me.SuspendLayout()
                Me.BorderPanel1.SuspendLayout()
                Me.BorderPanel1.PanelContents.SuspendLayout()

                For Each Ctl As Control In BorderPanel1.PanelContents.Controls
                    Dim Itm As IEditorControlItem = Ctl
                    RemoveHandler Itm.WasSelected, AddressOf Item_WasSelected
                    RemoveHandler Itm.ShouldBeDeleted, AddressOf Item_ShouldBeDeleted
                Next

                Me.BorderPanel1.PanelContents.Controls.Clear()
                SetSelected(Nothing)

                m_Source = value
                Me.PanelTop.Controls.Clear()
                If m_Source IsNot Nothing Then
                    If m_Source.CanSort Then
                        Me.UpDownPanel1.SortVisible = True
                        Me.UpDownPanel1.ButtonSort.Enabled = True
                    Else
                        Me.UpDownPanel1.SortVisible = False
                        Me.UpDownPanel1.ButtonSort.Enabled = False
                    End If

                    Dim Header As Control = m_Source.HeaderControl
                    Header.Dock = DockStyle.Fill
                    Me.PanelTop.Controls.Add(Header)

                    Dim Controls As New List(Of Control)
                    For Each DE As ChangeListItem In m_Source.GetItems

                        Dim Itm As Control = DE.CreateControl
                        Itm.SuspendLayout()
                        Itm.Dock = DockStyle.Top
                        Controls.Add(Itm)

                        Dim xCtl As IEditorControlItem = Itm
                        AddHandler xCtl.WasSelected, AddressOf Item_WasSelected
                        AddHandler xCtl.ShouldBeDeleted, AddressOf Item_ShouldBeDeleted
                    Next
                    Controls.Reverse()

                    Me.BorderPanel1.PanelContents.Controls.AddRange(Controls.ToArray)
                    SortTabStopps()
                    For Each Ctl As Control In Controls
                        Ctl.ResumeLayout()
                    Next
                End If
                Me.BorderPanel1.PanelContents.ResumeLayout()
                Me.BorderPanel1.ResumeLayout()
                Me.ResumeLayout()
            End If
        End Set
    End Property


    Protected Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
        MyBase.OnPaint(e)
        Dim Rect As Rectangle = New Rectangle(0, 0, Me.Width - 1, Me.Height - 1)
        ControlPaint.DrawVisualStyleBorder(e.Graphics, Rect)
    End Sub

    Public Sub SetSelectedControl(ByVal Value As ChangeListItem)
        For Each ICtl As IEditorControlItem In BorderPanel1.PanelContents.Controls
            If ICtl.Source Is Value Then
                Dim Ctl As Control = ICtl
                SetSelected(Ctl)
                Exit For
            End If
        Next
    End Sub
    Private m_SelectedControl As Control
    Private Sub SetSelected(ByVal SelectedControl As Control)
        If m_SelectedControl IsNot SelectedControl Then
            If m_SelectedControl IsNot Nothing Then
                m_SelectedControl.BackColor = SystemColors.Control
            End If
            m_SelectedControl = SelectedControl
            If m_SelectedControl IsNot Nothing Then
                m_SelectedControl.BackColor = SystemColors.Highlight
                Me.BorderPanel1.PanelContents.ScrollControlIntoView(m_SelectedControl)
            End If

            If m_SelectedControl Is Nothing Then
                Me.UpDownPanel1.ButtonMoveDown.Enabled = False
                Me.UpDownPanel1.ButtonMoveUp.Enabled = False
                Me.UpDownPanel1.ButtonSort.Enabled = False
            Else
                Dim SelCtl As IEditorControlItem = SelectedControl
                Me.UpDownPanel1.ButtonMoveUp.Enabled = m_Source.CanMoveUp(SelCtl.Source)
                Me.UpDownPanel1.ButtonMoveDown.Enabled = m_Source.CanMoveDown(SelCtl.Source)
                Me.UpDownPanel1.ButtonSort.Enabled = m_Source.CanSort
            End If

        End If
    End Sub
    Private Sub RefreshUpDown(ByVal SelCtl As IEditorControlItem)
        If SelCtl Is Nothing Then
            Me.UpDownPanel1.ButtonMoveUp.Enabled = False
            Me.UpDownPanel1.ButtonMoveDown.Enabled = False
        Else
            Me.UpDownPanel1.ButtonMoveUp.Enabled = m_Source.CanMoveUp(SelCtl.Source)
            Me.UpDownPanel1.ButtonMoveDown.Enabled = m_Source.CanMoveDown(SelCtl.Source)
        End If

    End Sub

    Private Sub Item_WasSelected(ByVal SelectedControl As IEditorControlItem)
        If Not m_EventsEnabled Then Exit Sub

        SetSelected(SelectedControl)
    End Sub

    Private Sub Item_ShouldBeDeleted(ByVal SelectedControl As IEditorControlItem)
        If m_Source.CanDelete(SelectedControl.Source) Then
            SetSelected(Nothing)
            RemoveHandler SelectedControl.WasSelected, AddressOf Item_WasSelected
            RemoveHandler SelectedControl.ShouldBeDeleted, AddressOf Item_ShouldBeDeleted
            m_Source.RemoveItem(SelectedControl.Source)
            BorderPanel1.PanelContents.Controls.Remove(SelectedControl)
        End If
    End Sub


    Private Sub UpDownPanel1_MoveDown() Handles UpDownPanel1.MoveDown
        m_EventsEnabled = False
        Dim SelCtl As IEditorControlItem = m_SelectedControl
        Dim Idx As Integer = BorderPanel1.PanelContents.Controls.GetChildIndex(SelCtl)
        BorderPanel1.PanelContents.Controls.SetChildIndex(SelCtl, Idx - 1)
        m_Source.MoveDown(SelCtl.Source)

        RefreshUpDown(SelCtl)
        SortTabStopps()
        Me.BorderPanel1.PanelContents.ScrollControlIntoView(SelCtl)
        m_EventsEnabled = True
    End Sub
    Private Sub UpDownPanel1_MoveUp() Handles UpDownPanel1.MoveUp
        m_EventsEnabled = False
        Dim SelCtl As IEditorControlItem = m_SelectedControl
        Dim Idx As Integer = BorderPanel1.PanelContents.Controls.GetChildIndex(SelCtl)
        BorderPanel1.PanelContents.Controls.SetChildIndex(SelCtl, Idx + 1)

        m_Source.MoveUp(SelCtl.Source)

        RefreshUpDown(SelCtl)
        SortTabStopps()
        Me.BorderPanel1.PanelContents.ScrollControlIntoView(SelCtl)
        m_EventsEnabled = True
    End Sub

    Private Sub UpDownPanel1_Sort() Handles UpDownPanel1.Sort
        m_EventsEnabled = False
        Dim SelCtl As IEditorControlItem = m_SelectedControl
        m_Source.SortList()
        Dim Liste As List(Of ChangeListItem) = m_Source.GetItems
        Dim Count As Integer = Liste.Count - 1
        For I As Integer = Liste.Count - 1 To 0 Step -1
            For Each Ctl As IEditorControlItem In BorderPanel1.PanelContents.Controls
                If Ctl.Source Is Liste(I) Then
                    BorderPanel1.PanelContents.Controls.SetChildIndex(Ctl, Count - I)
                    Exit For
                End If
            Next
        Next

        If SelCtl IsNot Nothing Then RefreshUpDown(SelCtl)
        SortTabStopps()
        If SelCtl IsNot Nothing Then Me.BorderPanel1.PanelContents.ScrollControlIntoView(SelCtl)
        m_EventsEnabled = True
    End Sub

   

    Private Sub m_Source_ItemAdded(ByVal NewItem As ChangeListItem) Handles m_Source.ItemAdded
        Dim Ctl As Control = NewItem.CreateControl
        Dim Itm As IEditorControlItem = Ctl
        Ctl.Dock = DockStyle.Top
        Me.BorderPanel1.PanelContents.Controls.Add(Ctl)
        Me.BorderPanel1.PanelContents.Controls.SetChildIndex(Ctl, 0)
        AddHandler Itm.WasSelected, AddressOf Item_WasSelected
        AddHandler Itm.ShouldBeDeleted, AddressOf Item_ShouldBeDeleted
        BorderPanel1.PanelContents.Refresh()

        Dim SelCtl As IEditorControlItem = m_SelectedControl
        RefreshUpDown(SelCtl)
        SortTabStopps()
    End Sub
    Private Sub m_Source_SpecialItemCreated(ByVal NewItem As ExpandibleItem) Handles m_Source.SpecialItemCreated
        Dim Ctl As Control = NewItem.CreateControl(True)
        Dim Itm As IEditorControlItem = Ctl
        Ctl.Dock = DockStyle.Top
        Me.BorderPanel1.PanelContents.Controls.Add(Ctl)
        Me.BorderPanel1.PanelContents.Controls.SetChildIndex(Ctl, 1)
        AddHandler Itm.WasSelected, AddressOf Item_WasSelected
        AddHandler Itm.ShouldBeDeleted, AddressOf Item_ShouldBeDeleted
        BorderPanel1.PanelContents.Refresh()

        SortTabStopps()

        Me.SetSelected(Ctl)
        'Dim SelCtl As IEditorControlItem = m_SelectedControl
        'RefreshUpDown(SelCtl)
        'SortTabStopps()
    End Sub
    Public Sub New()

        ' Dieser Aufruf ist fr den Windows Form-Designer erforderlich.
        InitializeComponent()

        ' Fgen Sie Initialisierungen nach dem InitializeComponent()-Aufruf hinzu.
        Me.BorderPanel1.PanelContents.AutoScroll = True
    End Sub

    Private Sub SortTabStopps()
        Dim Zahl As Integer = Me.BorderPanel1.PanelContents.Controls.Count - 1
        If Zahl > 0 Then
            For I As Integer = Zahl To 0 Step -1
                Me.BorderPanel1.PanelContents.Controls(I).TabIndex = Zahl - I
            Next
        End If
        'Me.BorderPanel1.PanelContents.Controls.SetChildIndex(Ctl, 0)
    End Sub

    Private m_Host As GroupBox
    Private m_LabelText As String = ""
    Private m_LabelColor As Color = Color.Black

    Public Sub Init(ByVal LabelText As String, ByVal LabelColor As System.Drawing.Color, ByVal Host As System.Windows.Forms.GroupBox)
        m_LabelText = LabelText
        m_LabelColor = LabelColor
        m_Host = Host
    End Sub
    Public ReadOnly Property Host() As System.Windows.Forms.GroupBox
        Get
            Return m_Host
        End Get
    End Property
    Public ReadOnly Property LabelColor() As System.Drawing.Color
        Get
            Return m_LabelColor
        End Get
    End Property
    Public ReadOnly Property LabelText() As String
        Get
            Return m_LabelText
        End Get
    End Property


End Class
