Public Class PatientStationsSelectorSource
    Inherits PatientSelectorSource


    Private m_Zimmer As New List(Of StationsZimmerItem)
    Private m_OhneBettZimmer As StationsZimmerItem
    Private m_Labels As New List(Of PatientLabel)

    Private m_StationsPatientItems As New List(Of StationsPatientItem)
    Private WithEvents m_Provider As PatientProvider
    ReadOnly Property OhneBettZimmer() As StationsZimmerItem
        Get
            Return m_OhneBettZimmer
        End Get
    End Property

    Public Sub New(ByVal SelectedLabel As PatientLabel)
        m_Provider = DataProvider.PatientProvider

        For Each Zi As Zimmer In DataProvider.Visiten.AktuelleVisite.ZugeordneteZimmer
            Dim Item As New StationsZimmerItem(Me, Zi.ZimmerNummer)
            m_Zimmer.Add(Item)
            For I As Integer = 1 To Zi.AnzahlBetten
                Item.Betten.Add(New StationsBettItem(Item, I))
            Next
        Next

        Dim MinDatum As New Date(1860, 1, 1)

        Dim Found As Boolean = False
        For Each Lbl As PatientLabel In m_Provider
            If Lbl.Entlassungsdatum < MinDatum Then
                Found = False
                For Each Item As StationsZimmerItem In m_Zimmer
                    If Item.Text.ToLower = Lbl.Zimmernummer Then
                        Dim BettFound As Boolean = False
                        For Each BItem As StationsBettItem In Item.Betten
                            If BItem.BettNr = Lbl.Bettnummer Then
                                Dim PItem As New StationsPatientItem(Lbl, BItem)
                                BItem.Patienten.Add(PItem)
                                m_StationsPatientItems.Add(PItem)

                                BettFound = True
                                Exit For
                            End If
                        Next
                        If Not BettFound Then
                            Dim LastBItem As StationsBettItem = Item.Betten(Item.Betten.Count - 1)
                            Dim PItem As New StationsPatientItem(Lbl, LastBItem)
                            LastBItem.Patienten.Add(PItem)
                            m_StationsPatientItems.Add(PItem)
                        End If

                        Found = True
                    End If
                Next

                If Not Found Then
                    If m_OhneBettZimmer Is Nothing Then
                        m_OhneBettZimmer = New StationsZimmerItem(Me, "Ohne Bett")
                        m_OhneBettZimmer.Betten.Add(New StationsBettItem(m_OhneBettZimmer, 1))
                    End If
                    Dim LastBettItem As StationsBettItem = m_OhneBettZimmer.Betten(0)
                    Dim PItem As New StationsPatientItem(Lbl, LastBettItem)
                    LastBettItem.Patienten.Add(PItem)
                    m_StationsPatientItems.Add(PItem)
                End If
                m_Labels.Add(Lbl)
            End If
        Next

        If m_OhneBettZimmer IsNot Nothing Then
            m_Zimmer.Add(m_OhneBettZimmer)
        End If

        Dim Comp As New PatientenBettIDComparer
        For Each ZIItem As StationsZimmerItem In m_Zimmer
            For Each BItem As StationsBettItem In ZIItem.Betten
                BItem.Patienten.Sort(Comp)
            Next
        Next

        ReOrder()
        SetSelectedWithoutEvents(SelectedLabel)
    End Sub
    Public Sub TryDeleteOhneBettZimmer()
        If m_OhneBettZimmer IsNot Nothing Then
            If m_OhneBettZimmer.Betten(0).Patienten.Count = 0 Then
                m_Zimmer.Remove(m_OhneBettZimmer)
                m_OhneBettZimmer = Nothing
            End If
        End If
    End Sub

    ReadOnly Property Zimmer() As List(Of StationsZimmerItem)
        Get
            Return m_Zimmer
        End Get
    End Property

    Public Overrides Sub RaiseOrderChanged()
        ReOrder()

        MyBase.RaiseOrderChanged()
    End Sub

    Private Sub ReOrder()
        Dim m_Items As PatientSelectorItemList = Me.Items
        m_Items.Clear()

        m_Items.Add(New DistanceItem(3))

        For Each Zi As StationsZimmerItem In m_Zimmer

            Zi.Items.Clear()

            For Each Bett As StationsBettItem In Zi.Betten
                If Bett.Patienten.Count = 0 Then
                    Zi.Items.Add(Bett)
                Else
                    For Each Pat As StationsPatientItem In Bett.Patienten
                        Zi.Items.Add(Pat)
                    Next
                End If
            Next

            m_Items.Add(Zi)
            For Each Itm As PatientSelectorItem In Zi.Items
                m_Items.Add(Itm)
            Next

            m_Items.Add(New DistanceItem(6))
        Next
    End Sub



    Public Overrides Function SetSelectedWithoutEvents(ByVal Lbl As PatientLabel) As Boolean
        For Each Itm As StationsPatientItem In m_StationsPatientItems
            If Itm.Label Is Lbl Then
                Me.SetInternSelected(Itm)
                Return True
            End If
        Next

        Return False
    End Function
    Public Overrides ReadOnly Property AllLabels() As PatientLabelGroupCollection
        Get
            Dim Result As New PatientLabelGroupCollection("")
            For Each Zi As StationsZimmerItem In m_Zimmer
                Dim Grp As New PatientLabelGroup(Zi.Text)
                Result.Add(Grp)
                For Each Bt As StationsBettItem In Zi.Betten
                    If Bt.Patienten.Count = 0 Then
                        Grp.Add(New LeerBettLabel)
                    Else
                        For Each Pat As StationsPatientItem In Bt.Patienten
                            Grp.Add(Pat.Label)
                        Next
                    End If
                Next
            Next

            Return Result

        End Get
    End Property
End Class
