Public Class Patient
    Implements IAdresse
    Private m_Label As PatientLabel
    Private WithEvents m_XMLPart As PatientXMLPart
    Public Event Changed()

    Public Function CreateClone() As Patient
        Dim NewXMLPart As PatientXMLPart = m_XMLPart.CreateClone
        Dim NewLabel As PatientLabel = m_Label.CreateClone
        Return New Patient(NewLabel, NewXMLPart)
    End Function
    Public Sub ReadValues(ByVal Original As Patient)
        m_Label.ReadValues(Original.Label)
        m_XMLPart.ReadValues(Original.XMLPart)
    End Sub
    ReadOnly Property Briefe() As List(Of String)
        Get
            Return m_XMLPart.Briefe
        End Get
    End Property
    Property Vorrangig() As Boolean
        Get
            Return m_XMLPart.Vorrangig
        End Get
        Set(ByVal value As Boolean)
            m_XMLPart.Vorrangig = value
        End Set
    End Property
    ReadOnly Property Medikamente() As MedikamentVerordnungsListe
        Get
            Return m_XMLPart.Medikamente
        End Get
    End Property
    ReadOnly Property Aerzte() As List(Of ArztLabel)
        Get
            Return m_XMLPart.Aerzte
        End Get
    End Property
    ReadOnly Property FreiTexte() As TextEintragsListe
        Get
            Return m_XMLPart.FreiTexte
        End Get
    End Property
    ReadOnly Property Diagnosen() As TextEintragsListe
        Get
            Return m_XMLPart.Diagnosen
        End Get
    End Property
    ReadOnly Property Aufgaben() As TextEintragsListe
        Get
            Return m_XMLPart.Aufgaben
        End Get
    End Property
    ReadOnly Property Befunde() As BefundTextEintragsListe
        Get
            Return m_XMLPart.Befunde
        End Get
    End Property
    ReadOnly Property Label() As PatientLabel
        Get
            Return m_Label
        End Get
    End Property
    ReadOnly Property XMLPart() As PatientXMLPart
        Get
            Return m_XMLPart
        End Get
    End Property
    Public Sub New(ByVal Label As PatientLabel, ByVal Patient As PatientXMLPart)
        m_Label = Label
        m_XMLPart = Patient
        m_Label.Geschlecht = m_XMLPart.Geschlecht
        Create_Kuerzel()
    End Sub
    Private Sub Create_Kuerzel()
        If m_XMLPart.Nachname = "" Then
            m_Label.KurzText = "Neuer Patient"
        Else
            Dim MinDatum As New Date(1860, 1, 1)
            Dim ZStr As String = m_XMLPart.Nachname
            If m_XMLPart.Vorname <> "" Then
                ZStr &= ", " & m_XMLPart.Vorname
            End If
            If m_XMLPart.Geburtsdatum > MinDatum Then
                Dim Alter As Integer = DateDiff(DateInterval.Year, m_XMLPart.Geburtsdatum, Date.Now)
                ZStr &= " " & Alter.ToString & "J."
            End If

            If m_XMLPart.Aufnahmedatum > MinDatum AndAlso m_Label.Entlassungsdatum < MinDatum Then
                Dim Tage As Integer = DateDiff(DateInterval.Day, m_XMLPart.Aufnahmedatum, Date.Now)
                ZStr &= " (" & Tage & ")"
            End If

            m_Label.KurzText = ZStr
        End If
    End Sub


    Private Sub m_XMLPart_AdressAnsicht_Geaendert() Handles m_XMLPart.AdressAnsicht_Geaendert
        RaiseEvent AdressAnsicht_Geaendert()
    End Sub

    Private Sub m_XMLPart_AnredeVorschlaege_Geaendert() Handles m_XMLPart.AnredeVorschlaege_Geaendert
        RaiseEvent AnredeVorschlaege_Geaendert()
    End Sub

    Private Sub m_XMLPart_BriefAnrede_AutomatischGeaendert() Handles m_XMLPart.BriefAnrede_AutomatischGeaendert
        RaiseEvent BriefAnrede_AutomatischGeaendert()
    End Sub

    Private Sub m_XMLPart_BriefAnrede_Geaendert() Handles m_XMLPart.BriefAnrede_Geaendert
        RaiseEvent BriefAnrede_Geaendert()
    End Sub

    Private Sub m_XMLPart_Geschlecht_Geaendert() Handles m_XMLPart.Geschlecht_Geaendert
        m_Label.Geschlecht = m_XMLPart.Geschlecht
        RaiseEvent Geschlecht_Geaendert()
    End Sub

    Private Sub m_XMLPart_Kuerzel_Geaendert() Handles m_XMLPart.Kuerzel_Geaendert
        RaiseEvent Kuerzel_Geaendert()
    End Sub
    'Private Sub m_XMLPart_LabelChanged(ByVal NewValue As String) Handles m_XMLPart.LabelChanged
    '    m_Label.KurzText = NewValue
    'End Sub


#Region " Properties "
    Public Event Kuerzel_Geaendert() Implements IAdresse.Kuerzel_Geaendert
    Public Event AdressAnsicht_Geaendert() Implements IAdresse.AdressAnsicht_Geaendert
    Public Event AnredeVorschlaege_Geaendert() Implements IAdresse.AnredeVorschlaege_Geaendert
    Public Event Geschlecht_Geaendert() Implements IAdresse.Geschlecht_Geaendert
    Public Event BriefAnrede_Geaendert() Implements IAdresse.BriefAnrede_Geaendert

    Public Event BriefAnrede_AutomatischGeaendert() Implements IAdresse.BriefAnrede_AutomatischGeaendert


    Property Aufnahmedatum() As Date
        Get
            Return m_XMLPart.Aufnahmedatum
        End Get
        Set(ByVal value As Date)
            If m_XMLPart.Aufnahmedatum <> value Then
                m_XMLPart.Aufnahmedatum = value
                Create_Kuerzel()
            End If
        End Set
    End Property
    Property Vorgeseheneentlassung() As Date
        Get
            Return m_XMLPart.Vorgeseheneentlassung
        End Get
        Set(ByVal value As Date)
            m_XMLPart.Vorgeseheneentlassung = value
        End Set
    End Property
    Property Nachname() As String Implements IAdresse.Nachname
        Get
            Return m_XMLPart.Nachname
        End Get
        Set(ByVal Value As String)
            If m_XMLPart.Nachname <> Value.Trim Then
                m_XMLPart.Nachname = Value.Trim
                Create_Kuerzel()
            End If
        End Set
    End Property
    Property Vorname() As String Implements IAdresse.Vorname
        Get
            Return m_XMLPart.Vorname
        End Get
        Set(ByVal Value As String)
            If m_XMLPart.Vorname <> Value.Trim Then
                m_XMLPart.Vorname = Value.Trim
                Create_Kuerzel()
            End If
        End Set
    End Property
    Property Ort() As String Implements IAdresse.Ort
        Get
            Return m_XMLPart.Ort
        End Get
        Set(ByVal Value As String)
            m_XMLPart.Ort = Value
        End Set
    End Property
    Property PLZ() As String Implements IAdresse.PLZ
        Get
            Return m_XMLPart.PLZ
        End Get
        Set(ByVal Value As String)
            m_XMLPart.PLZ = Value
        End Set
    End Property
    Property Strasse() As String Implements IAdresse.Strasse
        Get
            Return m_XMLPart.Strasse
        End Get
        Set(ByVal Value As String)
            m_XMLPart.Strasse = Value
        End Set
    End Property
    Property Firma() As String Implements IAdresse.Firma
        Get
            Return m_XMLPart.Firma
        End Get
        Set(ByVal Value As String)
            m_XMLPart.Firma = Value
        End Set
    End Property
    Property Firma1() As String Implements IAdresse.Firma1
        Get
            Return m_XMLPart.Firma1
        End Get
        Set(ByVal Value As String)
            m_XMLPart.Firma1 = Value
        End Set
    End Property
    Property Stellung() As String Implements IAdresse.Stellung
        Get
            Return m_XMLPart.Stellung
        End Get
        Set(ByVal Value As String)
            m_XMLPart.Stellung = Value
        End Set
    End Property
    Property Geburtsdatum() As Date Implements IAdresse.Geburtsdatum
        Get
            Return m_XMLPart.Geburtsdatum
        End Get
        Set(ByVal value As Date)
            If m_XMLPart.Geburtsdatum <> value Then
                m_XMLPart.Geburtsdatum = value
                Create_Kuerzel()
            End If
        End Set
    End Property
    Property FirmaVoran() As Boolean Implements IAdresse.FirmaVoran
        Get
            Return m_XMLPart.FirmaVoran
        End Get
        Set(ByVal Value As Boolean)
            m_XMLPart.FirmaVoran = Value
        End Set
    End Property
    Property Geschlecht() As Geschlechter Implements IAdresse.Geschlecht
        Get
            Return m_XMLPart.Geschlecht
        End Get
        Set(ByVal Value As Geschlechter)
            m_XMLPart.Geschlecht = Value
        End Set
    End Property
    Property Titel() As String Implements IAdresse.Titel
        Get
            Return m_XMLPart.Titel
        End Get
        Set(ByVal Value As String)
            m_XMLPart.Titel = Value
        End Set
    End Property
    Property Briefanrede() As String Implements IAdresse.Briefanrede
        Get
            Return m_XMLPart.Briefanrede
        End Get
        Set(ByVal Value As String)
            m_XMLPart.Briefanrede = Value
        End Set
    End Property

    ReadOnly Property FonFax() As FonFaxListe Implements IAdresse.FonFax
        Get
            Return m_XMLPart.FonFax
        End Get
    End Property

    Public Property AdressAnsicht() As String Implements IAdresse.AdressAnsicht
        Get
            Return m_XMLPart.AdressAnsicht
        End Get
        Set(ByVal value As String)

        End Set
    End Property

    Public Property Anredevorschlaege() As AnredeVorschlagsListe Implements IAdresse.Anredevorschlaege
        Get
            Return m_XMLPart.Anredevorschlaege
        End Get
        Set(ByVal value As AnredeVorschlagsListe)

        End Set
    End Property

    Public Property Kuerzel() As String Implements IAdresse.Kuerzel
        Get
            Return m_Label.KurzText
        End Get
        Set(ByVal value As String)

        End Set
    End Property

    Property Entlassungsdatum() As Date
        Get
            Return m_Label.Entlassungsdatum
        End Get
        Set(ByVal value As Date)
            If m_Label.Entlassungsdatum <> value Then
                m_Label.Entlassungsdatum = value
                Create_Kuerzel()
            End If
        End Set
    End Property
    Property Zimmernummer() As String
        Get
            Return m_Label.Zimmernummer
        End Get
        Set(ByVal value As String)
            m_Label.Zimmernummer = value
        End Set
    End Property
    Property Bettnummer() As Integer
        Get
            Return m_Label.Bettnummer
        End Get
        Set(ByVal value As Integer)
            m_Label.Bettnummer = value
        End Set
    End Property
    Property Erhalten() As Boolean
        Get
            Return m_Label.Erhalten
        End Get
        Set(ByVal value As Boolean)
            m_Label.Erhalten = value
        End Set
    End Property
#End Region

    Public Sub RaiseChanged()
        RaiseEvent Changed()
    End Sub
End Class

