Public Class PatientArea
    Inherits PrintArea
    Private m_Font As Font

    Private m_PatientNameGroup As ColorStringGroup
    'Private m_HauptDiagnoseGroup As ColorStringGroup

    Private m_AusstehendGroup As ColorStringLineGroup
    Private m_GesternHeuteMorgenGroup As ColorStringLineGroup

    Private m_DiagnosenGroup As ColorStringGroup
    Private m_BefundeGroup As ColorStringGroup

    Private m_FreiTextGroup As ColorStringGroup
    Private m_AufgabenGroup As ColorStringGroup

    Private m_Pat As Patient

    ReadOnly Property Pat() As Patient
        Get
            Return m_Pat
        End Get
    End Property
    Public Sub New(ByVal Pat As Patient)

        m_Pat = Pat
        m_Font = New Font("Arial", 8)

        m_PatientNameGroup = New ColorStringGroup(New Font("Arial", 10))
        m_PatientNameGroup.Add(Pat.Kuerzel)



        m_DiagnosenGroup = New ColorStringGroup(m_Font)
        m_BefundeGroup = New ColorStringGroup(m_Font)

        m_FreiTextGroup = New ColorStringGroup(m_Font)
        m_AufgabenGroup = New ColorStringGroup(m_Font)

        If Pat.Diagnosen.Count > 0 Then
            m_PatientNameGroup.Add(Pat.Diagnosen(0).Text)
            If Pat.Diagnosen.Count > 1 Then
                For I As Integer = 1 To Pat.Diagnosen.Count - 1
                    Dim Rw As TextEintrag = Pat.Diagnosen(I)
                    If Not Rw.IsEmpty Then
                        Dim Txt As String = Chr(183) & " " & Rw.Text
                        If Rw.Wichtig Then
                            m_DiagnosenGroup.Add(Txt, ColorLib.WarningColor)
                        Else
                            m_DiagnosenGroup.Add(Txt)
                        End If
                    End If
                Next
            End If
        End If

        Dim MinDatum As New Date(1860, 1, 1)

        For Each Rw As BefundTextEintrag In Pat.Befunde
            If Not Rw.IsEmpty Then
                Dim DatStr As String = ""
                If Rw.Datum < MinDatum Then
                    DatStr = ""
                Else
                    DatStr = " " & Format(Rw.Datum, "d.M.")
                End If
                If Rw.Wichtig Then
                    m_BefundeGroup.Add(Chr(183), ColorLib.WarningColor)
                    m_BefundeGroup.Add(Rw.Kategorie & DatStr, ColorLib.ZimmerColor)
                    m_BefundeGroup.Add(Rw.Text, ColorLib.WarningColor)
                Else
                    m_BefundeGroup.Add(Chr(183))
                    m_BefundeGroup.Add(Rw.Kategorie & DatStr, ColorLib.ZimmerColor)
                    m_BefundeGroup.Add(Rw.Text)
                End If
            End If
        Next

        For Each Rw As TextEintrag In Pat.Aufgaben
            If Not Rw.IsEmpty Then
                Dim Txt As String = Chr(183) & " " & Rw.Text
                If Rw.Wichtig Then
                    m_AufgabenGroup.Add(Txt, ColorLib.WarningColor)
                Else
                    m_AufgabenGroup.Add(Txt)
                End If
            End If
        Next


        m_FreiTextGroup.Add("Info:", ColorLib.NormalColor)
        For Each Rw As TextEintrag In Pat.FreiTexte
            If Not Rw.IsEmpty Then
                Dim Txt As String = Chr(183) & " " & Rw.Text.Trim
                If Rw.Wichtig Then
                    m_FreiTextGroup.Add(Txt, ColorLib.WarningColor)
                Else
                    m_FreiTextGroup.Add(Txt)
                End If
            End If
        Next


        CreateAusstehende(New AusstehendSammlung(Pat, DataProvider.BefundStichworte))
        CreateGesternHeuteMorgen(New GesternHeuteMorgenSammlung(Pat, DataProvider.BefundStichworte))

    End Sub
    Private Sub CreateGesternHeuteMorgen(ByVal Sammlung As GesternHeuteMorgenSammlung)
        m_GesternHeuteMorgenGroup = New ColorStringLineGroup(New Font("Arial", 6))

        If Sammlung.Gestern.Count > 0 Then
            m_AusstehendGroup.Add("Gestern:")
            For Each txt As String In Sammlung.Gestern
                If txt = "Entlassung" Then
                    m_AusstehendGroup.Add(" - " & txt, ColorLib.WarningColor1)
                Else
                    m_AusstehendGroup.Add(" - " & txt, ColorLib.WarningColor2)
                End If
            Next
        End If

        If Sammlung.Heute.Count > 0 Then
            m_AusstehendGroup.Add("Heute:")
            For Each txt As String In Sammlung.Heute
                If txt = "Entlassung" Then
                    m_AusstehendGroup.Add(" - " & txt, ColorLib.WarningColor1)
                Else
                    m_AusstehendGroup.Add(" - " & txt, ColorLib.WarningColor2)
                End If
            Next
        End If

        If Sammlung.Morgen.Count > 0 Then
            m_AusstehendGroup.Add("Morgen:")
            For Each txt As String In Sammlung.Morgen
                If txt = "Entlassung" Then
                    m_AusstehendGroup.Add(" - " & txt, ColorLib.WarningColor1)
                Else
                    m_AusstehendGroup.Add(" - " & txt, ColorLib.WarningColor2)
                End If
            Next
        End If



    End Sub

    Private Sub CreateAusstehende(ByVal Sammlung As AusstehendSammlung)
        m_AusstehendGroup = New ColorStringLineGroup(New Font("Arial", 6))
        If Sammlung.AusstehendePflichtBefunde.Count > 0 Then
            m_AusstehendGroup.Add("Pflicht:")
            For Each txt As String In Sammlung.AusstehendePflichtBefunde
                m_AusstehendGroup.Add(" - " & txt, ColorLib.WarningColor1)
            Next
        End If
        If Sammlung.LeerBefunde.Count > 0 Then
            m_AusstehendGroup.Add("Ausstehend:")
            For Each txt As String In Sammlung.LeerBefunde
                m_AusstehendGroup.Add(" - " & txt, ColorLib.WarningColor2)
            Next
        End If

    End Sub


    Private m_RandBreite As Single = 50.0F
    Public Overrides Function GetNeededHeight(ByVal g As System.Drawing.Graphics, ByVal Width As Single) As Single

        'Dim IconHeigt As Single = 16.0F
        'Dim IconLineWidth As Single = (IconHeigt + 2.0F) * 5.0F

        Dim NamensZeileHeight As Single = m_PatientNameGroup.GetHeight(g, Width - 50.0F)

        Dim SpaltenBreite As Single = (Width - m_RandBreite * 2) \ 2 - 10

        Dim DiagnosenGroupHeight As Single = m_DiagnosenGroup.GetHeight(g, SpaltenBreite)
        Dim BefundeGroupHeight As Single = m_BefundeGroup.GetHeight(g, SpaltenBreite)

        Dim FreiTextGroupHeight As Single = m_FreiTextGroup.GetHeight(g, SpaltenBreite)
        Dim AufgabenGroupHeight As Single = m_AufgabenGroup.GetHeight(g, SpaltenBreite)

        Dim Spalte1Height As Single = DiagnosenGroupHeight + FreiTextGroupHeight
        Dim Spalte2Height As Single = BefundeGroupHeight + AufgabenGroupHeight

        Dim MiddleHeight As Single = Math.Max(Spalte1Height, Spalte2Height)

        Dim AusstehendGroupHeight As Single = m_AusstehendGroup.GetHeight(g, 50.0F)
        Dim GesternHeuteMorgenGroupHeight As Single = m_GesternHeuteMorgenGroup.GetHeight(g, 50.0F)

        Dim MaxHeight As Single = Math.Max(AusstehendGroupHeight, GesternHeuteMorgenGroupHeight)
        MaxHeight = Math.Max(MiddleHeight, MaxHeight)
        MaxHeight += NamensZeileHeight
        Return Math.Max(MaxHeight, m_Font.GetHeight(g) * 5.0F)

    End Function

    Public Overrides Sub Print(ByVal g As System.Drawing.Graphics, ByVal Left As Single, ByVal Top As Single, ByVal Width As Single)

        'Dim IconHeigt As Single = 16.0F
        'Dim IconLineWidth As Single = (IconHeigt + 4.0F) * 5.0F

        Dim NamensZeileHeight As Single = m_PatientNameGroup.GetHeight(g, Width - 50.0F)


        Dim SpaltenBreite As Single = (Width - m_RandBreite * 2) \ 2 - 10

        Dim DiagnosenGroupHeight As Single = m_DiagnosenGroup.GetHeight(g, SpaltenBreite)
        Dim BefundeGroupHeight As Single = m_BefundeGroup.GetHeight(g, SpaltenBreite)

        Dim FreiTextGroupHeight As Single = m_FreiTextGroup.GetHeight(g, SpaltenBreite)
        Dim AufgabenGroupHeight As Single = m_AufgabenGroup.GetHeight(g, SpaltenBreite)

        Dim Spalte1Height As Single = DiagnosenGroupHeight + FreiTextGroupHeight
        Dim Spalte2Height As Single = BefundeGroupHeight + AufgabenGroupHeight

        Dim MiddleHeight As Single = Math.Max(Spalte1Height, Spalte2Height)

        Dim AusstehendGroupHeight As Single = m_AusstehendGroup.GetHeight(g, 50.0F)
        Dim GesternHeuteMorgenGroupHeight As Single = m_GesternHeuteMorgenGroup.GetHeight(g, 50.0F)

        Dim MaxHeight As Single = Math.Max(AusstehendGroupHeight, GesternHeuteMorgenGroupHeight)
        MaxHeight = Math.Max(MiddleHeight, MaxHeight)

        g.DrawLine(New Pen(ColorLib.ZimmerColor), Left + 25.0F, Top + NamensZeileHeight - 2.0F, Left + Width - 25.0F, Top + NamensZeileHeight - 2.0F)
        'g.FillRectangle(New SolidBrush(Color.White), New RectangleF(Left + Width - 25.0F - IconLineWidth, Top + 12.0F, IconLineWidth, 6.0F))

        m_PatientNameGroup.Print(g, Left + 25.0F, Top, Width - 25.0F)
        If m_Pat.Vorrangig Then
            g.FillEllipse(Brushes.Red, New Rectangle(Left + 5, Top + 5, 10, 10))
        End If

        m_AusstehendGroup.Print(g, Left, Top + NamensZeileHeight, 50.0F)
        m_GesternHeuteMorgenGroup.Print(g, Left + Width - 50.0F, Top + NamensZeileHeight, 50.0F)

        Dim DiagnosenLeft As Single = Left + 50.0F
        m_DiagnosenGroup.Print(g, DiagnosenLeft, Top + NamensZeileHeight, SpaltenBreite)
        Dim BefundeLeft As Single = DiagnosenLeft + SpaltenBreite + 5.0F
        m_BefundeGroup.Print(g, BefundeLeft, Top + NamensZeileHeight, SpaltenBreite)

        Dim FreiTextTop As Single = Top + NamensZeileHeight + DiagnosenGroupHeight
        m_FreiTextGroup.Print(g, DiagnosenLeft, FreiTextTop, SpaltenBreite)
        g.DrawLine(ColorLib.LinePen, DiagnosenLeft, FreiTextTop, DiagnosenLeft + SpaltenBreite, FreiTextTop)

        Dim AufgabenTop As Single = Top + NamensZeileHeight + BefundeGroupHeight
        m_AufgabenGroup.Print(g, BefundeLeft, AufgabenTop, SpaltenBreite)
        g.DrawLine(ColorLib.LinePen, BefundeLeft, AufgabenTop, BefundeLeft + SpaltenBreite, AufgabenTop)

        Dim RightPos As Single = Width - 25.0F + Left + 4.0F

        'Dim IconWidth As Single = IconHeigt + 4.0F



        Dim TextColor As Color = ColorLib.NormalColor

        'Dim PtrKHK As New KHKPainter(TextColor, New PointF(RightPos - IconWidth * 5.0F, Top), IconHeigt, m_Pat.Pflichtfragen)
        'Dim PtrNI As New NIPainter(TextColor, New PointF(RightPos - IconWidth * 4.0F, Top), IconHeigt, m_Pat.Pflichtfragen)
        'Dim PtrSD As New SDPainter(TextColor, New PointF(RightPos - IconWidth * 3.0F, Top), IconHeigt, m_Pat.Pflichtfragen)
        'Dim PtrAllerg As New AllergiePainter(TextColor, New PointF(RightPos - IconWidth * 2.0F, Top), IconHeigt, m_Pat.Pflichtfragen)
        'Dim PtrDiab As New DiabetesPainter(TextColor, New PointF(RightPos - IconWidth, Top), IconHeigt, m_Pat.Pflichtfragen)

        'PtrDiab.DrawImage(g)
        'PtrKHK.DrawImage(g)
        'PtrAllerg.DrawImage(g)
        'PtrNI.DrawImage(g)
        'PtrSD.DrawImage(g)

    End Sub
    'Private Sub DrawSizedImage(ByVal g As Graphics, ByVal Img As Image, ByVal Left As Single, ByVal Top As Single, ByVal Width As Single)
    '    If Img Is ImageLib.DisabledImage Then
    '        Dim NewImage As New Bitmap(CInt(Width), CInt(Width))
    '        Dim gr As Graphics = Graphics.FromImage(NewImage)
    '        gr.DrawImage(Img, New RectangleF(0, 0, Width, Width))
    '        ControlPaint.DrawImageDisabled(g, NewImage, Left, Top, Color.White)
    '    Else
    '        g.DrawImage(Img, New RectangleF(Left, Top, Width, Width))
    '    End If
    'End Sub
End Class

