Public Class RoentgenArea
    Inherits PrintArea
    'Private m_Patienten As List(Of Patient)
    Private m_Font As Font
    Private m_Patientennamen As New List(Of ColorStringGroup)
    Private m_ZimmerNummern As New List(Of ColorStringGroup)
    Private m_Diagnosen As New List(Of ColorStringGroup)

    Private m_Height As Single = 0.0F
    Private m_LineHeights As List(Of Single)
    Private m_NamenMaxWidth As Single = 0.0F

    Public Sub New()
        m_Font = New Font("Arial", 8)
        CreateStringGroups()
    End Sub
    Private Sub CreateStringGroups()

        Dim Alternator As New ColorAlternator(ColorLib.NormalColor, ColorLib.WarningColor)

        Dim AktuellePatienten As List(Of Patient) = DataProvider.PatientProvider.TempAktuellePatienten
        AktuellePatienten.Sort(New PatientNameComparer)

        For Each Pat As Patient In AktuellePatienten
            Dim Grp As New ColorStringGroup(m_Font)
            m_Patientennamen.Add(Grp)
            Grp.Add(Pat.Kuerzel, Alternator.NormalColor)

            Grp = New ColorStringGroup(m_Font)
            m_ZimmerNummern.Add(Grp)
            Grp.Add(Pat.Zimmernummer, Alternator.NormalColor)

            Grp = New ColorStringGroup(m_Font)
            m_Diagnosen.Add(Grp)

            For Each Rw As TextEintrag In Pat.Diagnosen
                If Not Rw.IsEmpty Then
                    Dim Txt As String = Rw.Text
                    If Rw.Wichtig Then
                        Grp.Add(Txt, Alternator.WarningColor)
                    Else
                        Grp.Add(Txt, Alternator.NormalColor)
                    End If
                End If

            Next

            Alternator.Alternate()
        Next
    End Sub



    Public Overrides Function GetNeededHeight(ByVal g As System.Drawing.Graphics, ByVal Width As Single) As Single
        If m_LineHeights Is Nothing Then
            m_Height = m_Font.GetHeight(g)

            m_LineHeights = New List(Of Single)

            Dim Maximum As Single = 10000.0F
            For Each grp As ColorStringGroup In m_Patientennamen
                m_NamenMaxWidth = Math.Max(grp.GetMaxWidth(g, Maximum), m_NamenMaxWidth)
            Next

            Dim DiagnosenWidth As Single = Width - 50.0F - m_NamenMaxWidth

            For I As Integer = 0 To m_Patientennamen.Count - 1
                Dim xHeight As Single = m_ZimmerNummern(I).GetHeight(g, 25.0F)
                'xHeight = Math.Max(xHeight, m_Patientennamen(I).GetHeight(g, 150.0F))
                xHeight = Math.Max(xHeight, m_Diagnosen(I).GetHeight(g, DiagnosenWidth))
                m_LineHeights.Add(xHeight)
                m_Height += xHeight
            Next
        End If

        Return m_Height

    End Function

    Public Overrides Sub Print(ByVal g As System.Drawing.Graphics, ByVal Left As Single, ByVal Top As Single, ByVal Width As Single)

        Dim xTop As Single = Top + m_Font.GetHeight(g)
        g.DrawString("Zusammenfassung", m_Font, New SolidBrush(ColorLib.ZimmerColor), Left, Top)

        Dim DiagnosenWidth As Single = Width - 50.0F - m_NamenMaxWidth

        Dim MustDrawLine As Boolean = True
        For I As Integer = 0 To m_Patientennamen.Count - 1

            If IsNumeric(m_ZimmerNummern(I).Strings(0).Text.Trim) Then
                m_ZimmerNummern(I).Print(g, Left + 25.0F, xTop, 25.0F)
            End If

            Dim Maximum As Single = 10000.0F
            m_Patientennamen(I).Print(g, Left + 50.0F, xTop, Maximum)

            m_Diagnosen(I).Print(g, Left + 50.0F + m_NamenMaxWidth, xTop, DiagnosenWidth)

            MustDrawLine = Not MustDrawLine
            If MustDrawLine Then
                g.DrawLine(ColorLib.LinePen, Left + 25.0F, xTop, Left + Width - 25.0F, xTop)
            End If

            xTop += m_LineHeights(I)
        Next

    End Sub
End Class
